pspy
====

A framework for privilege-separating Python apps.


Motivation
----------

Abiding to the principle of least privilege is cumbersome: you must figure out
how to launch each process, where to jail it, what user or group IDs it should
run under, and what permissions to set for all sensitive files. This leads to
long configuration files and scripts to bootstrap the environment, making
privilege separation not only hard, but also error prone.

Pspy removes this burden by providing a library for making it easier to
privilege-separate Python apps and an analysis tool that yields important
privilege information.


Install
-------

You need to have setuptools installed. Once you have that, go to the pspy
directory (where the pspy code is) and execute:

`sudo ./setup.py install`


Documentation
-------------

The main concept in pspy is that of a *service*. A service is an independent
unit of self-contained functionality. Every service is provided by one 
*service provider*. But a service provider can also provide multiple services.

Each service provider runs as a separate process, jailed, with a non-root
uid/gid. A socket is created per service provider, which is the socket that
clients will use to communicate with the services the service provider provides.

A service can also have:
* a specific group; and/or
* a list of additional groups; and/or
* a list of associated files with specific permissions.

To specify services in your app you use the service decorator:

```Python
import pspy

@pspy.service()
class BankService:
  ...
  def transfer(src, dst, amount):
    ...
```

Since no service provider was specified in the service annotation, this will
create a service provider that will only provide the `BankService`.

And then there's nothing special you need to do with the users of this service,
the following:

```Python

...
bs = BankService()
bs.transfer(src, dst, amount)
...
```

will work fine. At run-time this will end up making a call to the service
provider of the `BankService`.


To bootstrap the environment you run `pspy-bootstrap`. Then bootstrap
will parse your app's code detecting pspy services and starting them.

To run the analysis tool you execute `pspy-analyze`.


Examples
--------

In the pspy directory, there is a test_lib.py for simple testing. Run:

$ sudo pspy-bootstrap test_lib.py --jail-extra-files perms.txt

To run the test:

$ cd /jail
$ python test_lib.py

To run the analyzer:

$ sudo pspy-analyze test_lib.py


The following are more comprehensive examples:

* Zoobar

* Minitwit

* Dpaste

See the README files in the respective folders of the examples/ directory
for how to run each application with pspy.


Contributing
------------

If you want to hack on pspy then you should install it in develop mode:

`sudo ./setup.py develop`

This will create symlinks to the pspy Python files.
