import argparse
import lib
import os
import sys


class ServiceStructure:
  perms = {
    '0': '---',
    '1': '--x',
    '2': '-w-',
    '3': '-wx',
    '4': 'r--',
    '5': 'r-x',
    '6': 'rw-',
    '7': 'rwx'
  }

  def __init__(self, _name, _uid, _gid, _additional_gids):
    self.name = _name
    self.uid = _uid
    self.gid = _gid
    self.additional_gids = _additional_gids
    self.uid_files = {}
    self.gid_files = {}
    self.additional_gid_files = {}
    self.other_files = {}

  def displayInfo(self):
    print "\n**********     SERVICE     **********\n"
    print "NAME: ", self.name
    print "UID: ", self.uid
    print "GID: ", self.gid
    print "ADDITIONAL GIDS: ", self.additional_gids
    print "\nFILE PERMISSIONS AS USER:"
    for key in self.uid_files:
      print ServiceStructure.perms[str(self.uid_files[key])] + "\t" + key
    print "\nFILE PERMISSIONS AS GROUP:"
    for key in self.gid_files:
      print ServiceStructure.perms[str(self.gid_files[key])] + "\t" + key
    print "\nFILE PERMISSIONS AS ADDITIONAL GROUPS:"
    for key in self.additional_gid_files:
      print ServiceStructure.perms[str(self.additional_gid_files[key])] + "\t" + key
    print "\nFILE PERMISSIONS AS OTHER:"
    for key in self.other_files:
      print ServiceStructure.perms[str(self.other_files[key])] + "\t" + key
    print


def analyzeJail(config = lib.DEFAULT_CONFIG):
  service_structs = []
  service_providers = lib.parse_app(config.app_files)
  _id = int(config.min_uid)
  next_gid = int(config.min_gid)
  gids_dict = {}
  for service_provider in service_providers:
    services, group_name, permissions, add_group_names = service_providers[service_provider]
    next_gid, gids_dict, gid, groups = lib.get_gids(next_gid, gids_dict, group_name, add_group_names)
    s = ServiceStructure(service_provider, _id, gid, groups)
    service_structs.append(s)
    _id = _id +1

  def getAllFiles(d):
    files = []
    for f in os.listdir(d):
      full_path = os.path.join(d,f)
      break1 = False
      for e in sys.path:
        e1 = os.path.join(config.jail_dir, e[1:])
        if full_path.startswith(e1[:10]) or full_path == e1:
          break1 = True 
          break
      if break1:
        continue
      if os.path.isdir(full_path):
        files = files + getAllFiles(full_path)
      else:
        files.append(full_path)
    return files

  files = getAllFiles(config.jail_dir)
  for f in files:
    # run stat and add to ServiceStructures
    stat_info = os.stat(f)
    uid = stat_info.st_uid
    gid = stat_info.st_gid
    perms = oct(stat_info.st_mode & 0777)
    for s in service_structs:
      if s.uid == uid:
        s.uid_files[f] = perms[1]
      if s.gid == gid:
        s.gid_files[f] = perms[2]
      if gid in s.additional_gids:
        s.additional_gid_files[f] = perms[2]
      if str(perms[3]) != '0':
        s.other_files[f] = perms[3]
  print "\n---------------SERVICES---------------\n"
  for s in service_structs:
    s.displayInfo()


def main():
  parser = argparse.ArgumentParser(description='pspy analyze')
  parser.add_argument('app_files', nargs='*', default=lib.DEFAULT_CONFIG.app_files, help=(
        'set of filepatterns that make the webapp. These can be listed'
        'explicitly or a directory can be specified. Defaults to the cwd'))
  parser.add_argument('--jail-dir', default=lib.DEFAULT_CONFIG.jail_dir, help=(
    'the location where to put the jail. Defaults to /jail'))
  parser.add_argument('--min-uid', default=lib.DEFAULT_CONFIG.min_uid, help=(
    'the starting point of uids that are not used by the user. Defaults to 1000'))
  parser.add_argument('--min-gid', default=lib.DEFAULT_CONFIG.min_gid, help=(
    'the starting point of gids that are not used by the user. Defaults to 1000'))

  args = parser.parse_args()
  analyzeJail(config = lib.Config(args.jail_dir, args.app_files,
              lib.DEFAULT_CONFIG.jail_extra_files, args.min_uid,
              args.min_gid, lib.DEFAULT_CONFIG.script))


if __name__ == '__main__':
  main()
