import tempfile
import unittest

import pspy


@pspy.service(
    group='TestGroup', perms={'perms.txt': 0700},
    additional_groups=['Group1', 'Group2'])
class DummyService:

  def __init__(self):
    self.counter = 0

  def inc_counter(self):
    self.counter += 1

  def count(self):
    return self.counter


@pspy.service()
def hello(name):
  return 'hello ' + name


@pspy.service(service_provider='Counter')
class CounterService1:

  def __init__(self):
    self.counter = 0

  def inc_counter(self):
    self.counter += 1

  def count(self):
    return self.counter


@pspy.service(service_provider='Counter')
class CounterService2:

  def __init__(self):
    self.counter = 0

  def inc_counter(self):
    self.counter += 1

  def count(self):
    return self.counter


class TestServiceDecorator(unittest.TestCase):

  def test_basic_class(self):
    ds = DummyService()
    ds.inc_counter()
    self.assertEqual(1, ds.count())
    ds.inc_counter()
    self.assertEqual(2, ds.count())
    ds.inc_counter()
    self.assertEqual(3, ds.count())

  def test_basic_func(self):
    self.assertEqual('hello me', hello('me'))
    self.assertEqual('hello you', hello('you'))

  def test_provider(self):
    c1 = CounterService1()
    c2 = CounterService2()
    c1.inc_counter()
    c2.inc_counter()
    self.assertEqual(1, c1.count())
    self.assertEqual(1, c2.count())
    c1.inc_counter()
    c2.inc_counter()
    self.assertEqual(2, c1.count())
    self.assertEqual(2, c2.count())
    c1.inc_counter()
    c2.inc_counter()
    self.assertEqual(3, c1.count())
    self.assertEqual(3, c2.count())


if __name__ == '__main__':
  unittest.main()
