import tempfile
import unittest
import ast

from pspy import parser


HELLO = ("""
def hello(name):
  pass
""")

PYCODE_1 = ("""
import pspy

@pspy.service()
""")

PYCODE_2 = ("""
from pspy import *

@service()
""")

PYCODE_3 = ("""
import pspy

@pspy.service(
   group='Group', perms={'file': 0777}, additional_groups=['Group1', 'Group2'],
   service_provider='provider')
""")

PYCODE_4 = ("""
from pspy import *

@service(
    group='Group', perms={'file': 0777}, additional_groups=['Group1', 'Group2'],
    service_provider='provider')
""")


class TestParser(unittest.TestCase):

  def test_basic_func_1(self):
    self.__assert_basic(PYCODE_1 + HELLO)

  def test_basic_func_2(self):
    self.__assert_basic(PYCODE_2 + HELLO)

  def test_basic_func_3(self):
    self.__assert_kwargs(PYCODE_3 + HELLO)

  def test_basic_func_4(self):
    self.__assert_kwargs(PYCODE_4 + HELLO)

  def __assert_basic(self, pycode):
    result = parser.parse_string(pycode)
    self.assertEqual({'hello': (['hello'], '', {}, [])}, result)

  def __assert_kwargs(self, pycode):
    result = parser.parse_string(pycode)
    self.assertEqual(
        {'provider': (
            ['hello'], 'Group', {'file': 0777}, ['Group1', 'Group2'])},
        result)


if __name__ == '__main__':
  unittest.main()
